/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "accel2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define ACCEL2_DUMMY 0

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void accel2_i2c_write ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len );

static void accel2_i2c_read ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len );

static void accel2_spi_write ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len );

static void accel2_spi_read ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void accel2_cfg_setup ( accel2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->int_pin = HAL_PIN_NC;

    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = 0x1D;
    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
    cfg->sel = ACCEL2_MASTER_I2C;
}

ACCEL2_RETVAL accel2_init ( accel2_t *ctx, accel2_cfg_t *cfg )
{
    ctx->master_sel = cfg->sel;

    if ( ctx->master_sel == ACCEL2_MASTER_I2C )
    {
        i2c_master_config_t i2c_cfg;

        i2c_master_configure_default( &i2c_cfg );
        i2c_cfg.speed    = cfg->i2c_speed;
        i2c_cfg.pins.scl = cfg->scl;
        i2c_cfg.pins.sda = cfg->sda;

        ctx->slave_address = cfg->i2c_address;

        if (  i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
        {
            return ACCEL2_INIT_ERROR;
        }

        ctx->read_f = accel2_i2c_read;
        ctx->write_f = accel2_i2c_write;
    }
    else
    {
        spi_master_config_t spi_cfg;

        spi_master_configure_default( &spi_cfg );
        spi_cfg.mode        = cfg->spi_mode;
        spi_cfg.speed       = cfg->spi_speed;
        spi_cfg.pins.sck    = cfg->sck;
        spi_cfg.pins.miso   = cfg->miso;
        spi_cfg.pins.mosi   = cfg->mosi;
        spi_cfg.chip_select = cfg->cs;

        if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
        {
            return  ACCEL2_INIT_ERROR;
        }

        spi_master_set_dummy_data( &ctx->spi, ACCEL2_DUMMY );
        
        ctx->read_f = accel2_spi_read;
        ctx->write_f = accel2_spi_write;
    }

    // Input pins

    digital_in_init( &ctx->int_pin, cfg->int_pin );
    
    return ACCEL2_OK;
}

void accel2_generic_write ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    ctx->write_f( ctx, reg, data_buf, len ); 
}

void accel2_generic_read ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    ctx->read_f( ctx, reg, data_buf, len );
}

void accel2_setting ( accel2_t *ctx )
{
    uint8_t tmp;
    tmp = 0x77;
    accel2_generic_write( ctx, ACCEL2_CTRL_REG4, &tmp, 1 );
    tmp = 0xC8;
    accel2_generic_write( ctx, ACCEL2_CTRL_REG3, &tmp, 1 );
}

uint8_t accel2_check_id ( accel2_t *ctx )
{
    uint8_t id = 0x00;
    accel2_generic_read( ctx, ACCEL2_WHO_AM_I, &id, 1  );

    if ( id != 0x3F )
    {
        return 0x01;
    }
    else
    {
        return 0x00;
    }
}

uint16_t accel2_read_xaxis ( accel2_t *ctx )
{
    uint16_t out_x;
    uint8_t buffer[ 2 ];
    out_x = 0x0000;

    accel2_generic_read( ctx, ACCEL2_OUT_X_H, &buffer[ 0 ], 1 );
    accel2_generic_read( ctx, ACCEL2_OUT_X_L, &buffer[ 1 ], 1 );
    
    out_x = buffer[ 0 ];
    out_x <<= 8;
    out_x |= buffer[ 1 ];
    
    return out_x;
}

uint16_t accel2_read_yaxis ( accel2_t *ctx )
{
    uint16_t out_y;
    uint8_t buffer[ 2 ];
    out_y = 0x0000;

    accel2_generic_read( ctx, ACCEL2_OUT_Y_H, &buffer[ 0 ], 1 );
    accel2_generic_read( ctx, ACCEL2_OUT_Y_L, &buffer[ 1 ], 1 );

    out_y = buffer[ 0 ];
    out_y <<= 8;
    out_y |= buffer[ 1 ];

    return out_y;
}

uint16_t accel2_read_zaxis ( accel2_t *ctx )
{
    uint16_t out_z;
    uint8_t buffer[ 2 ];
    out_z = 0x0000;

    accel2_generic_read( ctx, ACCEL2_OUT_Z_H, &buffer[ 0 ], 1 );
    accel2_generic_read( ctx, ACCEL2_OUT_Z_L, &buffer[ 1 ], 1 );

    out_z = buffer[ 0 ];
    out_z <<= 8;
    out_z |= buffer[ 1 ];

    return out_z;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void accel2_i2c_write ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg;
    
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = data_buf[ cnt - 1 ]; 
    }
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = len + 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );    
}

static void accel2_i2c_read ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 1 ];

    tx_buf [ 0 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

static void accel2_spi_write ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    spi_master_transfer_data_t block;

    uint8_t tx_buf[ 265 ];
    uint8_t cnt;

    tx_buf[ 0 ] = reg;
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = data_buf[ cnt - 1 ]; 
    }
    
    block.tx_buffer = tx_buf;
    block.rx_buffer = 0;
    block.tx_length = len + 1;
    block.rx_length = 0;

    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, &block);
    spi_master_stop(&ctx->spi);    
}

static void accel2_spi_read ( accel2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    spi_master_transfer_data_t block;

    uint8_t tx_buf[ 1 ];
    uint8_t rx_buf[ 265 ];
    uint8_t cnt;

    tx_buf[ 0 ] = reg | 0x80;
    
    block.tx_buffer = tx_buf;
    block.rx_buffer = rx_buf;
    block.tx_length = 1;
    block.rx_length = len + 1;
    
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, &block);
    spi_master_stop(&ctx->spi);   

    for ( cnt = 0; cnt < len; cnt++ )
    {
        data_buf[ cnt ] = rx_buf [ cnt + 1];
    }
}

// ------------------------------------------------------------------------- END

